% This program implements  the Case Study
% CS_Project_Selection_FXCHG with mpsg_solver  
%------------------------------------------------------------------------
%Solving Problem 1 (linear)
%Clear workspace 
clc
clear;

global psg_suppress_message;
psg_suppress_message = true;

%Load data
load('CS_Project_Selection_FXCHG_tbpsg_run_data.mat');
tbpsg_export_to_workspace(toolboxstruc_arr);

%Specify the set of values of upper bound on available initial capital
UB_Init_Capital = [300 350 400 500 600 650];
Objectives(6) =0;

%Begin loop for Optimization Problem 1 over parameter values  
for i=1:1:length(UB_Init_Capital)

%Generate problem statement
    clear problem_statement;
    problem_statement = sprintf('%s\n',...
    'maximize',...
    '  linear(matrix_CS_Project_Selection_npv)',...
    ['Constraint: constraint_budget_linear, <=  ',num2str(UB_Init_Capital(i)),', linearize = 1'],...
    '  linear(matrix_CS_Project_Selection_costs)',...
    'Box: >= point_lowerbounds, <= point_upperbounds , types = 1',...
    '');

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

% Call the solver
[solution_str, outargstruc_arr] = tbpsg_run(problem_statement, toolboxstruc_arr);
    
%Extract optimal solution   
    point_variables = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);
    Points_Problem_1(:,i) = tbpsg_optimal_point_data(solution_str, outargstruc_arr)';
    Objectives(i) = tbpsg_objective(solution_str, outargstruc_arr); 
    constr_function_value = tbpsg_constraints_data(solution_str, outargstruc_arr);   

end

%Prepare output values and display them in the Command Window
project_name = point_variables;
fprintf('\n%12s','Table 1: Solution of Problem 1 (linear)');
fprintf('\n%12s','UB_Init_Capital');
for i=1:1:length(UB_Init_Capital)
    fprintf('%12.0f', UB_Init_Capital(i));
end
fprintf('\n%5s','Objective        ');
for i=1:1:length(UB_Init_Capital)
    fprintf('%12.4f', Objectives(i));
end
fprintf('\n%5s','Optimal points');
fprintf('\n%5s','Project');
for j=1:size(Points_Problem_1,1)
    fprintf('\n%8s', project_name{j});
    fprintf('        ');
   for i=1:1:length(UB_Init_Capital)
    fprintf('%12f', Points_Problem_1(j,i));
   end
end 
fprintf('\n'); fprintf('\n');  

%Create plots for output
subplot(2,1,1);
  plot2 = plot(UB_Init_Capital, Objectives, '-ob', 'LineWidth', 1.5, 'MarkerFaceColor', 'b', 'MarkerSize', 3);
            title('Objective vs Parameter: linear', 'FontSize', 12);
            xlabel('Parameter');
            ylabel('Objectives');
            grid on;

%--------------------------------------------------------------------------
%Solving Problem 2 (fxchg_pos)          
%Clear workspace 
clear;
mpsg_suppress_message('On'); % PSG messages are swithced off
mpsg_suppress_warning('On'); % PSG warnings are swithced off

%load data
load('CS_Project_Selection_FXCHG_tbpsg_run_data.mat');
tbpsg_export_to_workspace(toolboxstruc_arr);

%Specify the set of values of upper bound on available initial capital
UB_Init_Capital = [300 350 400 500 600 650];
Objectives(6) =0;

%Begin loop for Optimization Problem 2 over parameter values  
for i=1:length(UB_Init_Capital)

%Generate problem statement
    clear problem_statement;
    problem_statement = sprintf('%s\n',...
    'maximize',...
    '  linear(matrix_cs_project_selection_npv)',...
    ['Constraint: constraint_budget, <=  ',num2str(UB_Init_Capital(i)),', linearize = 1'],...
    '  fxchg_pos(0.0000001, matrix_cs_project_selection_costs)',...
    'Box: >= point_lowerbounds, <= point_upperbounds',...   
    '');

%Uncomment the following line to open the problem in Toolbox Window:
%tbpsg_toolbox(problem_statement,toolboxstruc_arr);

% Call the solver
[solution_str, outargstruc_arr] = tbpsg_run(problem_statement, toolboxstruc_arr);
    
%Extract optimal solution   
    point_variables = tbpsg_optimal_point_vars(solution_str, outargstruc_arr);
    Points_Problem_2(:,i) = tbpsg_optimal_point_data(solution_str, outargstruc_arr)';
    Objectives(i) = tbpsg_objective(solution_str, outargstruc_arr); 
    constr_function_value = tbpsg_constraints_data(solution_str, outargstruc_arr);   

end

%Prepare output values and display them in the Command Window
project_name = point_variables;
fprintf('\n%12s','Table 2: Solution of Problem 2 (fxchg_pos)');
fprintf('\n');
fprintf('\n%12s','UB_Init_Capital');
for i=1:1:length(UB_Init_Capital)
    fprintf('%12.0f', UB_Init_Capital(i));
end

fprintf('\n%5s','Objective        ');
for i=1:1:length(UB_Init_Capital)
    fprintf('%12.4f', Objectives(i));
end
fprintf('\n%5s','Optimal points');
fprintf('\n%5s','Project');
for j=1:size(Points_Problem_2,1)
    fprintf('\n%8s', project_name{j});
    fprintf('        ');
   for i=1:1:length(UB_Init_Capital)
    fprintf('%12f', Points_Problem_2(j,i));
   end
end 
fprintf('\n'); fprintf('\n'); 

%Create plots for output
subplot(2,1,2);
            plot1 = plot(UB_Init_Capital, Objectives, '-ob', 'LineWidth', 1.5, 'MarkerFaceColor', 'b', 'MarkerSize', 3);
            title('Objective vs Parameter: fxchg\_pos', 'FontSize', 12);
            xlabel('Parameter');
            ylabel('Objectives');
            grid on;
       
%=======================================================================|
%American Optimal Decisions, Inc. Copyright                             |
%Copyright American Optimal Decisions, Inc. 2007-2016.                 |
%American Optimal Decisions (AOD) retains copyrights to this material.  |
%                                                                       |
%Permission to reproduce this document and to prepare derivative works  |
%from this document for internal use is granted, provided the copyright |
%and No Warranty  statements are included with all reproductions       |
%and derivative works.                                                  |
%For information regarding external or commercial use of copyrighted    |
%materials owned by AOD, contact AOD at support@aorda.com.              |
%=======================================================================|